package company

import (
	"github.com/gin-gonic/gin"
	"hongze/hongze_yb/controller/response"
	"hongze/hongze_yb/services/company"
	userService "hongze/hongze_yb/services/user"
)

// GetPermissionList 获取FICC品种权限列表
// @Tags 客户模块
// @Summary  获取FICC品种权限列表
// @Description 获取FICC品种权限列表
// @Security ApiKeyAuth
// @Param Authorization	header string true "Bearer 31a165baebe6dec616b1f8f3207b4273"
// @Accept  json
// @Product json
// @Success 200 {object} []company.FiccPermissionList
// @failure 400 {string} string "获取失败"
// @Router /company/GetPermissionList [get]
func GetPermissionList(c *gin.Context)  {
	//wechartUser := user.GetInfoByClaims(c)

	list, err := company.GetFiccPermissionList()
	if err != nil {
		response.FailMsg("获取失败", "获取FICC品种权限列表失败, Err:" + err.Error(), c)
		return
	}

	//userInfo, err, _ := userLogic.GetUserInfo(wechartUser)
	//if err != nil {
	//	response.FailMsg("获取失败", "获取用户信息失败, Err:" + err.Error(), c)
	//	return
	//}

	response.OkData("获取成功", list, c)
}


func GetHomeFiccPermissions(c *gin.Context) {
	userinfo := userService.GetInfoByClaims(c)

	list, err := company.GetHomeFiccPermissions(userinfo)
	if err !=nil {
		response.Fail(err.Error(), c)
		return
	}

	response.OkData("获取成功", list, c)
}