package response

import (
	"time"
)

type ReportDetail struct {
	ReportInfo          *ReportItem  `json:"report_info"`
	ReportChapterList        []*ReportChapterListItem `json:"report_chapter_list"`
	PermissionCheck     *PermissionCheckInfo    `json:"permission_check"`
	AuthOk              bool `json:"auth_ok"`
}

type ReportChapterListItem struct {
	ReportChapterId    int    `json:"report_chapter_id"`
	ReportId        int    `json:"report_id"`
	Title           string    `json:"title"`
	TypeId                  int         `json:"type_id"`
	TypeName                string         	`json:"type_name"`
	Trend                   string      `json:"trend"`
	ReportChapterTypeKey    string      `json:"report_chapter_type_key"`
	ReportChapterTypeThumb  string	    `json:"report_chapter_type_thumb"`
	ReportChapterTypeName   string       `json:"report_chapter_type_name"`
	Sort                    int          `json:"sort"`
	PublishTime         time.Time    `json:"publish_time"`
}

type ReportItem struct {
	ReportId           int    `json:"report_id"`
	ClassifyNameFirst  string `description:"一级分类名称" json:"classify_name_first"`
	ClassifyNameSecond string `description:"二级分类名称" json:"classify_name_second"`
	Title              string `description:"标题" json:"title"`
	Abstract           string `description:"摘要" json:"abstract"`
	Author             string `description:"作者" json:"author"`
	Frequency          string `description:"频度" json:"frequency"`
	PublishTime        time.Time `description:"发布时间" json:"publish_time"`
	Stage              int    `description:"期数" json:"stage"`
	Content            string `description:"内容" json:"content"`
	VideoUrl           string `description:"音频文件URL" json:"video_url"`
	VideoName          string `description:"音频文件名称" json:"video_name"`
	VideoPlaySeconds   string `description:"音频播放时长" json:"video_play_seconds"`
	ContentSub         string `description:"内容前两个章节" json:"content_sub"`
	BannerUrl          string `description:"详情页banner" json:"banner_url"`
}

type ReportChapterItem struct {
	ReportChapterId    int    `json:"report_chapter_id"`
	ReportId        int    `json:"report_id"`
	Title           string    `json:"title"`
	TypeId                  int         `json:"type_id"`
	TypeName                string         	`json:"type_name"`
	Trend                   string      `json:"trend"`
	ReportChapterTypeName   string       `json:"report_chapter_type_name"`
	PublishTime         time.Time    `json:"publish_time"`
	Content            string `description:"内容" json:"content"`
	ContentSub         string `description:"内容前两个章节" json:"content_sub"`
	VideoUrl            string    `json:"video_url"`                    //音频文件URL
	VideoName           string    `json:"video_name"`                  //音频文件名称
	VideoPlaySeconds    string    `json:"video_play_seconds"`      //音频播放时长
	VideoSize           string    `json:"video_size"`
	Author              string    `description:"作者" json:"author"`
	Stage              int        `description:"期数" json:"stage"`
	ClassifyIdFirst    int        `description:"一级分类id" json:"classify_id_first"`
	ClassifyNameFirst  string     `description:"一级分类名称" json:"classify_name_first"`
}

type ReportChapterDetail struct {
	ReportChapterItem  * ReportChapterItem  `json:"report_chapter_item"`
	PermissionCheck     *PermissionCheckInfo    `json:"permission_check"`
	ReportChapterMenuList        []*ReportChapterMenu `json:"report_chapter_menu_list""`
	AuthOk              bool `json:"auth_ok"`
}

type ReportChapterMenu struct {
	ReportChapterId    int    `json:"report_chapter_id"`
	ReportId        int    `json:"report_id"`
	ReportChapterTypeName   string       `json:"report_chapter_type_name"`
	ReportChapterTypeThumb  string	    `json:"report_chapter_type_thumb"`
}

type ReportListItem struct {
	ReportId                 int    `description:"报告Id" json:"report_id"`
	ClassifyIdFirst    int    `description:"一级分类id" json:"classify_id_first"`
	ClassifyNameFirst  string `description:"一级分类名称" json:"classify_name_first"`
	ClassifyIdSecond   int    `description:"二级分类id" json:"classify_id_second"`
	ClassifyNameSecond string `description:"二级分类名称" json:"classify_name_second"`
	Title              string `description:"标题" json:"title"`
	Abstract           string `description:"摘要" json:"abstract"`
	Author             string `description:"作者" json:"author"`
	ReportImgUrl             string `description:"作者头像" json:"report_img_url"`
	PublishTime        time.Time `description:"发布时间" json:"publish_time"`
	Stage              int    `description:"期数" json:"stage"`
	VideoUrl           string `description:"音频文件URL" json:"video_url"`
	AuthOk              bool `json:"auth_ok"`
}

type ReportList struct {
	Paging     *PagingItem  		`json:"paging"`
	List        []*ReportListItem `json:"list"`
}

type ReportCollectListItem struct {
	ReportId                 int    `description:"报告Id" json:"report_id"`
	ReportChapterId    int    `json:"report_chapter_id"`
	ClassifyIdFirst    int    `description:"一级分类id" json:"classify_id_first"`
	ClassifyNameFirst  string `description:"一级分类名称" json:"classify_name_first"`
	ClassifyIdSecond   int    `description:"二级分类id" json:"classify_id_second"`
	ClassifyNameSecond string `description:"二级分类名称" json:"classify_name_second"`
	ReportChapterTypeId                  int         `json:"report_chapter_type_id"`
	PublishTime        time.Time `description:"发布时间" json:"publish_time"`
	Title              string `description:"标题" json:"title"`
	ContentSub         string `description:"内容前两个章节" json:"content_sub"`
}

type ReportCollectList struct {
	Date     string   	`json:"date"`
	SubList  []*ReportCollectListItem `json:"sub_list"`
}

type ReportCollectResp struct {
	Paging     *PagingItem  		`json:"paging"`
	List []*ReportCollectList `json:"list"`
}

type ReportSearchResp struct {
	Paging     *PagingItem  		`json:"paging"`
	List []*ReportCollectListItem `json:"list"`
}

type EsReportItem struct {
	Author             string `description:"作者"`
	BodyContent        string `description:"内容"`
	Categories         string `description:"品种名称"`
	ClassifyIdFirst    int    `description:"一级分类id"`
	ClassifyNameFirst  string `description:"一级分类名称"`
	ClassifyIdSecond   int    `description:"二级分类id"`
	ClassifyNameSecond string `description:"二级分类名称"`
	PublishState       int    `description:"1:未发布，2:已发布"`
	PublishTime        string `description:"发布时间"`
	ReportChapterId    int    `description:"报告章节Id"`
	ReportId           int    `description:"报告Id"`
	Title              string `description:"标题"`
}

type TickerDataItem struct {
	Date             string        `json:"date"`
	Ticker			 string        `json:"ticker"`
	BaseColumnName   string        `json:"base_column_name"`
	TickerValue      float64       `json:"ticker_value"`
	LastValue        float64       `json:"last_value"`
	DdValue          float64       `json:"dd_value"`
	WwValue          float64       `json:"ww_value"`
	MmValue          float64       `json:"mm_value"`
}

type ReportCollectListResp []*ReportCollectList

func (p ReportCollectListResp) Len() int {
	return len(p)
}

func (p ReportCollectListResp) Less(i, j int) bool {
	return p[i].Date > p[j].Date
}

func (p ReportCollectListResp) Swap(i, j int) {
	p[i], p[j] = p[j], p[i]
}