package elastic

import (
	"context"
	"errors"
	"fmt"
	"github.com/olivere/elastic/v7"
	"hongze/hongze_yb/global"
	"hongze/hongze_yb/utils"
)

func SearchReport(keyWord string, pageIndex, pageSize int) (searchResp *elastic.SearchResult, total int64, err error) {
	indexName := global.CONFIG.EsClient.Prefix+utils.ES_INDEX_RDDP_REPORT
	var must  []map[string]interface{}

	shouldSub := []map[string]interface{}{
		map[string]interface{}{
			"match": map[string]interface{}{
				"Title": map[string]interface{}{
					"query": keyWord,
				//"minimum_should_match": "60%",
					"boost": 3,
				},
			},
		},
		map[string]interface{}{
			"match": map[string]interface{}{
				"Categories": map[string]interface{}{
					"query": keyWord,
					//"minimum_should_match": "60%",
					"boost": 2,
				},
			},
		},
		map[string]interface{}{
			"match": map[string]interface{}{
				"BodyContent": map[string]interface{}{
					"query": keyWord,
					//"minimum_should_match": "60%",
					"boost": 1,
				},
			},
		},
		map[string]interface{}{
			"match_phrase": map[string]interface{}{
				"Title": map[string]interface{}{
					"query": keyWord,
					//"slop": "50",
					"boost": 5,
				},
			},
		},
		map[string]interface{}{
			"match_phrase": map[string]interface{}{
				"Categories": map[string]interface{}{
					"query": keyWord,
					"boost": 4,
				},
			},
		},
		map[string]interface{}{
			"match_phrase": map[string]interface{}{
				"BodyContent": map[string]interface{}{
					"query": keyWord,
					"boost": 3,
				},
			},
		},
	}
	mustMap := map[string]interface{}{
		"bool": map[string]interface{}{
			"should": shouldSub,
		},
	}
	must = append(must, mustMap)

	must = append(must, map[string]interface{}{
		"match": map[string]interface{}{
			"PublishState": 2,
		},
	})
	source := map[string]interface{}{
		"query": map[string]interface{}{
			"bool": map[string]interface{}{
				"must": must,
			},
		},
	}
	source["from"] = (pageIndex - 1) * pageSize
	source["size"] = pageSize
	source["highlight"] = map[string]interface{}{
		"fields": map[string]interface{}{
			"Title":map[string]interface{}{},
			"Categories":map[string]interface{}{},
			"BodyContent":map[string]interface{}{},
		},
		"pre_tags" : "<span style=\"color:#efefef\">",
		"post_tags": "</span>",
	}

	source["sort"] = []map[string]interface{}{
		map[string]interface{}{
			"_score":map[string]interface{}{
				"order":"desc",
			},
		},
		map[string]interface{}{
			"PublishTime.keyword":map[string]interface{}{
				"order":"desc",
			},
		},
	}
	fmt.Println(source)
	/*highlight := elastic.NewHighlight()
	highlight = highlight.Fields(elastic.NewHighlighterField("content"), elastic.NewHighlighterField("title"))
	highlight = highlight.PreTags("<font color='red'>").PostTags("</font>")*/
	request := global.EsClient.Search(indexName).Source(source) // sets the JSON request

//	searchMap := make(map[string]string)

	searchResp, err = request.Do(context.Background())
	if err != nil {
		fmt.Print("结果err：")
		fmt.Println(err.Error())
		return
	}

	fmt.Print("结果正常：")
	fmt.Println(searchResp.Hits)
	fmt.Println(searchResp.Status)
	if searchResp.Status != 0 {
		err = errors.New("查询失败")
	}
	total = searchResp.TotalHits()
	return
}