package purchase

import (
	"errors"
	"hongze/hongze_yb/models/response/purchase"
	activity2 "hongze/hongze_yb/services/activity"
	"hongze/hongze_yb/services/company"
	"hongze/hongze_yb/services/report"
	"hongze/hongze_yb/services/user"
	"sort"
)

func GetLatestReportAndActivity(user user.UserInfo) (list purchase.PurchaseList, err error)  {
	permissionIds, err := company.GetPurchasePermissionIdsByCompany2ProductId(user.CompanyID, 1)
	if err != nil {
		return
	}
	if len(permissionIds) == 0 {
		err = errors.New("用户无权限")
		return
	}
	list, err = report.GetLatestClassReport(permissionIds, user.UserID)
	if err != nil {
		return
	}

	purItem, err := activity2.GetLatestActivity(permissionIds, user.UserID)
	if err != nil {
		return
	}

	list = append(list, purItem)

	if len(list) > 0 {
		sort.Sort(list)
	}
	return
}

// GetDetail 已购详情页面
func GetDetail(user user.UserInfo, classifyIdFirst int, activityId, pageIndex, pageSize int) (ret *purchase.DetailResp, err error)  {
	permissionIds, err := company.GetPurchasePermissionIdsByCompany2ProductId(user.CompanyID, 1)
	if err != nil {
		return
	}
	if len(permissionIds) <= 0 {
		err = errors.New("用户无权限")
		return
	}

	if classifyIdFirst != 0 {
		ret, err = report.GetPurchaseDetail(permissionIds, user.UserID, classifyIdFirst, pageIndex, pageSize)
	}else if activityId > 0 {
		ret, err = activity2.GetPurchaseDetail(permissionIds, user.UserID, pageIndex, pageSize)
	}
	return
}

